/*
 * Copyright (c) 1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project. <http://java.apache.org/>"
 *
 * 4. The names "Java Apache Element Construction Set", "Java Apache ECS" and 
 *    "Java Apache Project" must not be used to endorse or promote products 
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called 
 *    "Java Apache Element Construction Set" nor "Java Apache ECS" appear 
 *    in their names without prior written permission of the 
 *    Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project. <http://java.apache.org/>"
 *    
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.ecs;

import java.util.Hashtable;
import java.io.PrintWriter;
import java.io.OutputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;


/**
    This class implements the ElementFactory.

    @version $Id: Element.java,v 1.7 1999/05/18 00:52:59 jonbolt Exp $
    @author <a href="mailto:snagy@servletapi.com">Stephan Nagy</a>
    @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
*/
public abstract class Element implements ElementFactory
{

    /** Filter to use to escape input */
    private Filter filter = new CharacterFilter();

    /** position of tag relative to start and end. */
    private int position = ElementFactory.CENTER;
    
    /** Should we filter the value of &lt;&gt;VALUE&lt;/&gt; */
    private boolean filter_state = true;

    /** Version number for codebase */
    private static final String VERSION = "1.0a3";

    /** Default value to set case type */
    private int case_type = MIXEDCASE;

    /** Where to get the element definition from */
    private Hashtable element = new Hashtable();

    /** Does this element need a closing tag? */
    private boolean end_element = true;

    /** What codeset are we going to use the default is 8859_1 */
    private String codeset = "8859_1";

    /** Base class for defining all elements. */
    public Element()
    {
    }

    /** Get the Hashtable representation of an element. */
    protected Hashtable getElementHashEntry()
    {
        return(element);
    }

    /**
        set the filter state of the element.
        @param filter_state (true|false) do we want to use a filter? default true;
    */
    public void setFilterState(boolean filter_state)
    {
        this.filter_state = filter_state;
    }

    /**
        get the filter state of the element.
    */
    public boolean getFilterState()
    {
        return(filter_state);
    }

    /** Get the filter that is currently in use. */
    public Filter getFilter()
    {
        return(filter);
    }

    /** Override the default filter with a user supplied filter. */
    public void setFilter(Filter filter)
    {
        this.filter = filter;
    }

    /** What version of htmlGen is this? */
    public String getVersion()
    {
        return(VERSION);
    }

    /** Determine what case the html element should be rendered. Default is MIXEDCASE */
    public void setCase(int case_type)
    {
    this.case_type = case_type;
    }

    /** Find out what case the element is supposed to be rendered in.*/
    public int getCase()
    {
        return(case_type);
    }

    /** Determine what type of element this is A,BR,EM,ACRONYM */
    public void setElementType(String element_type)
    {
        element.put(ElementAttributes.TAG,element_type);
    }

    /** Find out what type of element this is. */
    public String getElementType()
    {
        return((String) element.get(ElementAttributes.TAG));
    }

    /** Determine what codeset should be used. */
    public void setCodeSet(String codeset)
    {
        this.codeset = codeset;
    }

    /** Find out what the current codeset is. */
    public String getCodeSet()
    {
        return(codeset);
    }

    /** Does this element need a closing tag. Default is true */
    public void setNeedClosingTag(boolean close)
    {
        end_element = close;
    }

    /** Determine if this element needs to be closed. */
    public boolean getNeedClosingTag()
    {
        return(end_element);
    }

    public void setTagPosition(int position)
    {
        this.position = position;
    }

    public int getTagPosition()
    {
        return(position);
    }

    /**     Alter the case of this element so that it is rendered according
            to whatever the value of case_type is.  Default is MIXEDCASE */
    protected String alterCase(String value)
    {
        switch (getCase())
        {
            case UPPERCASE:
                return value.toUpperCase();
            case LOWERCASE:
                return value.toLowerCase();
            default:
                return value;
        }
    }

    /** Override toString() method to print something meaningful */
    public final String toString()
    {
       String out = new String();
       try
        {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            BufferedOutputStream bos = new BufferedOutputStream(baos);
            output(bos);
            bos.flush();
            out = baos.toString();
            bos.close();
            baos.close();
        }
        catch (IOException ioe)
        {
        }
        return(out);
    }

    /** Override toString(encode) method to print something meaningful */
    public final String toString(String codeset)
    {

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        BufferedOutputStream bos = new BufferedOutputStream(baos);
        String out = new String();
        try
        {
            output(bos);
            bos.flush();
            out = baos.toString(codeset);  // XXXXX:  there has got to be a better way.
            bos.close();
            baos.close();
        }
        catch (UnsupportedEncodingException use)
        {
        }
        catch (IOException ioe)
        {
        }
        return(out);
    }

    public abstract void output(OutputStream out);
    public abstract void output(PrintWriter out);
}